﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace DCS_2000E_Wave
{
    public partial class Form_Wave : Form
    {

        private System.Net.Sockets.TcpClient m_TcpClient;           // TCP Client
        private System.Net.Sockets.NetworkStream m_NetworkStream;   // Stream

        const int iwordsize = 25000;                                //Record Length for DSO 10k

        public Form_Wave()
        {
            InitializeComponent();

            txtIP_1.Text = "";
            txtIP_2.Text = "";
            txtIP_3.Text = "";
            txtIP_4.Text = "";
            txtIP_PORT.Text = "3000";
            btnCONNECT.Text = "Connect";
            btnReadWave.Enabled = false;
            btnReadPict.Enabled = false;
        }

        private void btnCONNECT_Click(object sender, EventArgs e)
        {

            // Get IPv4 addr
            string strIpAddress = txtIP_1.Text + "." + txtIP_2.Text + "." + txtIP_3.Text + "." + txtIP_4.Text;
            if (btnCONNECT.Text == "Connect")       //Open
            {
                txtReadBuf.Text = "";

                if (SocketOpen(strIpAddress, txtIP_PORT.Text) == true)
                {
                    System.Threading.Thread.Sleep(100);
                    				                    // IDN read
                    SocketSendData("*IDN?\n");
                    System.Threading.Thread.Sleep(500);
                    SocketReadString();
                    if (txtReadBuf.Text != "")          // Retry 
                    {
                        System.Threading.Thread.Sleep(500);
                        SocketReadString();
                    }
                    if (txtReadBuf.Text != "")          // Connected
                    {
                        btnCONNECT.Text = "Disconnect";
                        btnReadWave.Enabled = true;
                        btnReadPict.Enabled = true;

                        //SocketSendData(":acq:reco " + iwordsize.ToString() +"\n");      //Set Record Length to DSO
                        //SocketSendData(":HARD:SAVEFORM bmp\n");                         //Set  bmp file mode

                    }
                    else
                    {
                        SocketClose();                  //close
                    }
                }
                else
                {
                    txtReadBuf.Text = "Open error";
                    SocketClose();                  //close
                }
            }
            else
            {
                btnCONNECT.Text = "Connect";
                SocketClose();
                btnReadWave.Enabled = false;
                btnReadPict.Enabled = false;
                txtReadBuf.Text = "";
            }

        }

        // Read Wave  data
        private void btnReadWave_Click(object sender, EventArgs e)
        {
            SocketSendData("*cls\n");
            System.Threading.Thread.Sleep(100);
            SocketSendData(":acq1:mem?\n");
            System.Threading.Thread.Sleep(500);
            SocketReadString();                     //Header Read       
            SocketReadBinary((iwordsize * 2) + 2 + (int)(Math.Log(iwordsize)/2.3));
            if (chkSaveOption.Checked == true)
            {
                SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                saveFileDialog1.Title = "Save Wave data to File";
                saveFileDialog1.Filter = "TEXT File|*.txt|All File|*.*";
                saveFileDialog1.OverwritePrompt = true;
                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    System.IO.StreamWriter writer = new System.IO.StreamWriter(saveFileDialog1.FileName, false, Encoding.GetEncoding("Shift_JIS"));
                    writer.Write(txtReadBuf.Text);
                    writer.Write(txtReadWave.Text);
                    writer.Close();
                }
                saveFileDialog1.Dispose();
            }
        }

        // Read Picture data
        private void btnReadPict_Click(object sender, EventArgs e)
        {
            this.txtReadBuf.Text = "";
            SocketSendData("disp:outp?\n");
            System.Windows.Forms.Application.DoEvents();
            System.Threading.Thread.Sleep(100);
            SocketReadBinaryPict();
        }

        //***********************************************************************/
        // DCS/GDS Socket Access
        //***********************************************************************/

        //Open
        private Boolean SocketOpen(string strIpAddress, string IPv4_Port)
        {
            this.m_NetworkStream = null;
            this.m_TcpClient = null;

            try
            { 
                this.m_TcpClient = new System.Net.Sockets.TcpClient();
                this.m_TcpClient.Connect(strIpAddress, Convert.ToInt32(IPv4_Port));
                this.m_NetworkStream = this.m_TcpClient.GetStream();
                this.m_TcpClient.ReceiveBufferSize = 100000 * 51 / 10  ;
                
                return (true);
            }
            catch (Exception ex)
            {
                this.m_NetworkStream = null;
                this.m_TcpClient = null;
                return (false);
            }
        }

        // Close
        private void SocketClose()
        {
            // Connected?
            if (this.m_TcpClient != null && this.m_TcpClient.Connected == true)
            {
                this.m_NetworkStream.Close();
                this.m_TcpClient.Close();
            }

            // free object 
            this.m_NetworkStream = null;
            this.m_TcpClient = null;
        }

        // Send string
        private void SocketSendData(string buf)
        {
            if (0 < buf.Length)
            {
                // encoding
                Byte[] DataBuff = System.Text.Encoding.GetEncoding("SHIFT-JIS").GetBytes(buf);

                // Send
                this.m_NetworkStream.Write(DataBuff, 0, DataBuff.GetLength(0));
                this.m_NetworkStream.Flush();
            }
        }

        // Recive string
        private void SocketReadString()
        {
            int DataCnt = this.m_TcpClient.Available;
            int resSize = 0;
            Byte[] DataBuff = new Byte[DataCnt];
            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            System.Text.Encoding enc = System.Text.Encoding.UTF8;

            if (DataCnt > 0)
            {
                //ReadLine
                do
                {
                    resSize = this.m_NetworkStream.Read(DataBuff, 0, 1);
                    ms.Write(DataBuff, 0, 1);

                } while (DataBuff[0] != '\n');
                this.txtReadBuf.Text = enc.GetString(ms.GetBuffer(), 0, (int)ms.Length);
                ms.Close();
            }
        }

        private void SocketReadBinary(int requestCnt)
        {
            int resSize = 0;
            int iHeadLength;
            string tmpstr = "";
            int iTemp;
            int[] sbuf = new int[requestCnt / 2+1];
            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            System.Text.Encoding enc = System.Text.Encoding.UTF8;
//            while (this.m_TcpClient.Available < (requestCnt))  { System.Threading.Thread.Sleep(100);  }
            for (int i = 0;i < 10; i ++) System.Threading.Thread.Sleep(100);

            if (this.m_TcpClient.Available >= (requestCnt))
            {
                Byte[] DataBuff = new Byte[requestCnt + 2];
                // Binary Header read
                resSize = this.m_NetworkStream.Read(DataBuff, 0, 2);
                ms.Write(DataBuff, 0, 2);

                iHeadLength = DataBuff[1] - 48;
                resSize = this.m_NetworkStream.Read(DataBuff, 0, iHeadLength );
                ms.Write(DataBuff, 0, iHeadLength );
                this.txtReadBuf.Text += enc.GetString(ms.GetBuffer(), 0, iHeadLength + 2 );

                ms.Close();

                // Binary Data read
                resSize = this.m_NetworkStream.Read(DataBuff, 0, requestCnt - iHeadLength);
                tmpstr = "";
                for (int index = 0; index < iwordsize; index++)
                {
                    // Endian convert (byteswap)
                    iTemp = DataBuff[index * 2];
                    DataBuff[index * 2] = DataBuff[index * 2 + 1];
                    DataBuff[index * 2 + 1] = (byte)iTemp;
                    sbuf[index] = BitConverter.ToInt16(DataBuff, index * 2);
                    tmpstr = tmpstr + "\r\n" + sbuf[index];
//                    System.Diagnostics.Debug.WriteLine(index.ToString() + " ," + sbuf[index].ToString());
                }

                txtReadWave.Text = tmpstr + "\r\n";

                //Create object BMP,Graphics 
                Bitmap canvas = new Bitmap(pictureBox1.Width, pictureBox1.Height);
                Graphics g = Graphics.FromImage(canvas);
                g.DrawLine(Pens.Orange, 0, 0, 0, 130);
                g.DrawLine(Pens.Orange, 0, 0, 250, 0);
                g.DrawLine(Pens.Orange, 250, 130, 250, 0);
                g.DrawLine(Pens.Orange, 250, 130, 0, 130);
                g.Dispose();

                //Plot to image
                Color c = Color.OrangeRed;
                int lim;

                for (int index = 0; index < (iwordsize - 2); index++)
                {
                    lim = (130 - sbuf[index]) / 2;
                    if (lim < 0) lim = 0;
                    if (lim > 130) lim = 130;
                    canvas.SetPixel((index * 250)/ iwordsize, lim, c);
                }
                pictureBox1.Image = canvas;
                if (this.m_TcpClient.Available > 0)
                    resSize = this.m_NetworkStream.Read(DataBuff, 0, this.m_TcpClient.Available);
            }
        }


        private void SocketReadBinaryPict()
        {
            int resSize = 0;
            int iHeadLength;

            int requestCnt = 10 ;
            Byte[] DataBuff1 = new Byte[requestCnt + 2];

            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            System.Text.Encoding enc = System.Text.Encoding.UTF8;
            while (this.m_TcpClient.Available < (requestCnt))
            {
                System.Threading.Thread.Sleep(100);
            }

            if (this.m_TcpClient.Available >= (requestCnt))
            {
                // Binary Header read
                resSize = this.m_NetworkStream.Read(DataBuff1, 0, 2);
                ms.Write(DataBuff1, 0, 2);

                iHeadLength = DataBuff1[1] - 48;
                resSize = this.m_NetworkStream.Read(DataBuff1, 0, iHeadLength);
                ms.Write(DataBuff1, 0, iHeadLength);
                //this.txtReadBuf.Text = enc.GetString(ms.GetBuffer(), 0, iHeadLength + 2);

                ms.Close();
                requestCnt = Convert.ToInt32(enc.GetString(ms.GetBuffer(), 2, iHeadLength));
                Byte[] DataBuff = new Byte[requestCnt + 2];

                while (this.m_TcpClient.Available > requestCnt)
                    resSize = this.m_NetworkStream.Read(DataBuff, 0, requestCnt);
                while (this.m_TcpClient.Available > 0)
                    resSize = this.m_NetworkStream.Read(DataBuff1, 0, this.m_TcpClient.Available);

                //Create object BMP,Graphics 
                Bitmap canvas = new Bitmap(800, 480);
 
                //Plot to image
                Color c  ;
                int iXpoint = 0;
                int iYpoint = 0;
                int iCount = 0;
                int iStep = 0;
                int iBlue, iRed, iGreen,iColor;


                for (int index = 0; index < requestCnt; index += 4)
                {
                    iStep = DataBuff[index] + (DataBuff[index + 1] * 256);
                    iColor = DataBuff[index + 2] + (DataBuff[index + 3] * 256);
                    iBlue  = ((iColor & 0x1F)+1) * 8 - 1;
                    iGreen = (((iColor >> 5) & 0x3f)+1) * 4 - 1;
                    iRed = (((iColor >> 11) & 0x1f)+1) * 8 - 1;
                    c = Color.FromArgb(iRed, iGreen, iBlue);
                    for(iCount = 0; iCount < iStep ; iCount ++)
                    {
                        canvas.SetPixel(iXpoint, iYpoint, c);
                        iXpoint ++;
                        if (iXpoint >= 800)
                        {
                            iXpoint = 0;
                            iYpoint++;
                        }
                    }
                }

                pictureBox1.Image = canvas;
                if (chkSaveOption.Checked == true)
                {
                    SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                    saveFileDialog1.Title = "Save Image to File";
                    saveFileDialog1.Filter = "PNG File|*.png|All File|*.*";
                    saveFileDialog1.OverwritePrompt = true;
                    if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                    {
                        canvas.Save(saveFileDialog1.FileName, System.Drawing.Imaging.ImageFormat.Png);
                    }
                    saveFileDialog1.Dispose();
                }
                this.txtReadBuf.Text += System.DateTime.Now + "\r\n";
            }
        }


    }
}
